package api

import (
	"fmt"
	"sku3dweb/bus"
	"sku3dweb/db/model"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"infish.cn/comm"
)

func RegTreeAssetOps(router *GinRouter) {
	router.POSTJWT("/asset/usersubmit/team", usersubmit2team)

	router.POSTJWT("/asset/usersubmit/company", usersubmit2company)

	router.POSTJWT("/asset/teamsubmit/company", teamsubmit2company)

	router.POSTJWT("/asset/clone/company", clonecompany2self)
	router.POSTJWT("/asset/clone/team", cloneteam2self)

}

type SubmitBody struct {
	AssetType string
	Id        string
}

// 提交到部门
func usersubmit2team(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	body := &SubmitBody{}
	err := c.ShouldBindJSON(body)
	if err != nil {
		return nil, err
	}

	isCompany := model.USERTYPE_COMPANY == apictx.User.UserType
	isSku3d := apictx.User.Parent == model.SKU3D_USERID
	//from collection 2
	srcColl := getAssetTypeCollectin(isCompany, isSku3d, "user", body.AssetType)

	//target collection
	targetColl := getAssetTypeCollectin(isCompany, isSku3d, "team", body.AssetType)

	//userInfo

	user, err := bus.GetUserInfo(apictx.User.ID)
	if err != nil {
		return nil, err
	}
	userInfo := &comm.AssetUserInfo{Name: user.Name, Thumbnail: &comm.OssType{Url: user.Avatar}}

	//获取当前用户的所属部门
	//ownerType = "team"
	//ownerId = "xx"

	if isAssetPack(body.AssetType) {

		pack, err := bus.BusTreeAssetPackDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}
		pack.OwnerId, _ = primitive.ObjectIDFromHex(user.TeamId)
		pack.OwnerType = "team"
		pack.UserInfo = userInfo
		pack.Id = primitive.NilObjectID
		return bus.BusTreeAssetPackAdd(Sku3dDbName, targetColl, pack)

	} else if isFabric3d(body.AssetType) {

		group, err := bus.BusTreeAssetMatgroupDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}
		group.OwnerId, _ = primitive.ObjectIDFromHex(user.TeamId)
		group.OwnerType = "team"
		group.UserInfo = userInfo
		group.Id = primitive.NilObjectID

		return bus.BusTreeAssetMatgroupAdd(Sku3dDbName, targetColl, group)
	} else if isAssetImage(body.AssetType) {

		group, err := bus.BusTreeAssetImageDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}
		group.OwnerId, _ = primitive.ObjectIDFromHex(user.TeamId)
		group.OwnerType = "team"
		group.UserInfo = userInfo
		group.Id = primitive.NilObjectID

		return bus.BusTreeAssetImageAdd(Sku3dDbName, targetColl, group)
	}

	return nil, fmt.Errorf("不支持的类型")
}

func usersubmit2company(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	body := &SubmitBody{}
	err := c.ShouldBindJSON(body)
	if err != nil {
		return nil, err
	}

	isCompany := model.USERTYPE_COMPANY == apictx.User.UserType
	isSku3d := apictx.User.Parent == model.SKU3D_USERID
	//from collection 2
	srcColl := getAssetTypeCollectin(isCompany, isSku3d, "user", body.AssetType)

	//target collection
	targetColl := getAssetTypeCollectin(isCompany, isSku3d, "company", body.AssetType)

	//userInfo

	user, err := bus.GetUserInfo(apictx.User.ID)
	if err != nil {
		return nil, err
	}
	userInfo := &comm.AssetUserInfo{Name: user.Name, Thumbnail: &comm.OssType{Url: user.Avatar}}

	if isAssetPack(body.AssetType) {

		pack, err := bus.BusTreeAssetPackDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}

		pack.OwnerId, _ = primitive.ObjectIDFromHex(apictx.User.Parent) //所在的企业ID
		pack.OwnerType = "company"
		pack.UserInfo = userInfo
		pack.Id = primitive.NilObjectID
		falseVale := false
		pack.Enable = &falseVale

		return bus.BusTreeAssetPackAdd(Sku3dDbName, targetColl, pack)
	} else if isFabric3d(body.AssetType) {

		pack, err := bus.BusTreeAssetMatgroupDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}

		pack.OwnerId, _ = primitive.ObjectIDFromHex(apictx.User.Parent) //所在的企业ID
		pack.OwnerType = "company"
		pack.UserInfo = userInfo
		pack.Id = primitive.NilObjectID
		falseVale := false
		pack.Enable = &falseVale

		return bus.BusTreeAssetMatgroupAdd(Sku3dDbName, targetColl, pack)
	} else if isAssetImage(body.AssetType) {

		pack, err := bus.BusTreeAssetImageDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}

		pack.OwnerId, _ = primitive.ObjectIDFromHex(apictx.User.Parent) //所在的企业ID
		pack.OwnerType = "company"
		pack.UserInfo = userInfo
		pack.Id = primitive.NilObjectID
		falseVale := false
		pack.Enable = &falseVale

		return bus.BusTreeAssetImageAdd(Sku3dDbName, targetColl, pack)
	}

	return nil, fmt.Errorf("不支持的类型")
}

func teamsubmit2company(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	body := &SubmitBody{}
	err := c.ShouldBindJSON(body)
	if err != nil {
		return nil, err
	}

	isCompany := model.USERTYPE_COMPANY == apictx.User.UserType
	isSku3d := apictx.User.Parent == model.SKU3D_USERID
	//from collection 2
	srcColl := getAssetTypeCollectin(isCompany, isSku3d, "team", body.AssetType)

	//target collection
	targetColl := getAssetTypeCollectin(isCompany, isSku3d, "company", body.AssetType)

	//userInfo

	user, err := bus.GetUserInfo(apictx.User.ID)
	if err != nil {
		return nil, err
	}
	userInfo := &comm.AssetUserInfo{Name: user.Name, Thumbnail: &comm.OssType{Url: user.Avatar}}

	if isAssetPack(body.AssetType) {

		pack, err := bus.BusTreeAssetPackDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}

		pack.OwnerId, _ = primitive.ObjectIDFromHex(apictx.User.Parent) //所在的企业ID
		pack.OwnerType = "company"
		pack.UserInfo = userInfo
		pack.Id = primitive.NilObjectID
		falseVale := false
		pack.Enable = &falseVale

		return bus.BusTreeAssetPackAdd(Sku3dDbName, targetColl, pack)
	} else if isFabric3d(body.AssetType) {

		pack, err := bus.BusTreeAssetMatgroupDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}

		pack.OwnerId, _ = primitive.ObjectIDFromHex(apictx.User.Parent) //所在的企业ID
		pack.OwnerType = "company"
		pack.UserInfo = userInfo
		pack.Id = primitive.NilObjectID
		falseVale := false
		pack.Enable = &falseVale

		return bus.BusTreeAssetMatgroupAdd(Sku3dDbName, targetColl, pack)
	} else if isAssetImage(body.AssetType) {

		pack, err := bus.BusTreeAssetImageDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}

		pack.OwnerId, _ = primitive.ObjectIDFromHex(apictx.User.Parent) //所在的企业ID
		pack.OwnerType = "company"
		pack.UserInfo = userInfo
		pack.Id = primitive.NilObjectID
		falseVale := false
		pack.Enable = &falseVale

		return bus.BusTreeAssetImageAdd(Sku3dDbName, targetColl, pack)
	}

	return nil, nil
}

func isAssetPack(name string) bool {
	return name == "shoes" || name == "last" || name == "heel" || name == "sole" || name == "shoe" || name == "decorate" || name == "shoe2d"
}

func isShoePack(name string) bool {
	return name == "shoes" || name == "shoe"
}

func isFabric3d(name string) bool {
	return name == "fabric3d" || name == "fabric2d" || name == "fabric" || name == "imagemat"
}

func isOtherMat(name string) bool {
	return name == "specialmat" || name == "linemat"
}

func isEnv3d(name string) bool {
	return name == "env"
}

func isAssetImage(name string) bool {
	return name == "sticker"
}

func clone2self(c *gin.Context, apictx *ApiSession, fromScope string) (interface{}, error) {

	body := &SubmitBody{}
	err := c.ShouldBindJSON(body)
	if err != nil {
		return nil, err
	}

	isCompany := model.USERTYPE_COMPANY == apictx.User.UserType
	isSku3d := apictx.User.Parent == model.SKU3D_USERID
	//from collection 2
	srcColl := getAssetTypeCollectin(isCompany, isSku3d, fromScope, body.AssetType)

	//target collection
	targetColl := getAssetTypeCollectin(isCompany, isSku3d, "user", body.AssetType)

	//userInfo

	user, err := bus.GetUserInfo(apictx.User.ID)
	if err != nil {
		return nil, err
	}
	userInfo := &comm.AssetUserInfo{Name: user.Name, Thumbnail: &comm.OssType{Url: user.Avatar}}

	if isAssetPack(body.AssetType) {
		pack, err := bus.BusTreeAssetPackDetail(Sku3dDbName, srcColl, body.Id)
		if err != nil {
			return nil, err
		}

		pack.OwnerId = user.Id
		pack.OwnerType = "user"
		pack.UserInfo = userInfo
		pack.UserId = user.Id
		pack.Id = primitive.NilObjectID
		falseVale := false
		pack.Enable = &falseVale
		return bus.BusTreeAssetPackAdd(Sku3dDbName, targetColl, pack)
	}
	return nil, fmt.Errorf("不支持的类型")
}

func clonecompany2self(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return clone2self(c, apictx, "company")
}

func cloneteam2self(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return clone2self(c, apictx, "team")
}
