package api

import (
	"bytes"
	"encoding/json"
	"errors"
	"io/ioutil"
	"net/http"
	"sku3dweb/bus"
	"sku3dweb/conf"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"infish.cn/comm"
)

func ServiceTaskList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	designId := c.Param("id")
	if len(designId) < 1 {
		return nil, NewError("id不能为空")
	}
	page, size, _ := UtilQueryPageSize(c)

	id, _ := primitive.ObjectIDFromHex(designId)

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionTask,
		Query:       repo.Map{"assetId": id},
		Project:     []string{"runState", "format", "runPercent", "runResult", "createTime", "cameras.thumbnail", "pack.name"},
		Page:        page,
		Size:        size,
		Sort:        bson.M{"createTime": -1},
	})
	// w := c.Writer
	// header := w.Header()
	// header.Set("Transfer-Encoding", "chunked")
	// header.Set("Content-Type", "text/plain")
	// w.WriteHeader(http.StatusOK)

	// //Flush()方法，好比服务端在往一个文件中写了数据，浏览器会看见此文件的内容在不断地增加。
	// w.Write([]byte(`0:render\n`))
	// w.(http.Flusher).Flush()

	// for i := 0; i < 100; i++ {
	// 	w.Write([]byte(fmt.Sprintf("%d:handle%d\n", i, i+1)))
	// 	w.(http.Flusher).Flush()
	// 	time.Sleep(time.Duration(1) * time.Second)
	// }

	// w.Write([]byte(`100:ok`))

	// w.(http.Flusher).Flush()

	// return nil, nil

}

func ServiceTaskDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	taskId := c.Param("id")
	if len(taskId) < 1 {
		return nil, NewError("id不能为空")
	}
	_, ret := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionTask,
		Project:     []string{"runState", "runPercent", "runResult", "createTime", "cameras.thumbnail", "pack.name"},
		Query:       repo.Map{"_id": taskId},
	})
	return ret, nil
}

func ServiceTaskRemove(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	taskId := c.Param("id")
	if len(taskId) < 1 {
		return nil, NewError("id不能为空")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionTask, taskId)
}

func ServiceTaskCreate(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	taskConfig := &comm.TreeRenderTask{CreateTime: time.Now()}
	err := c.ShouldBindJSON(taskConfig)
	if err != nil {
		return nil, err
	}

	taskId, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionTask, taskConfig)
	if err != nil {
		return nil, err
	}

	err = bus.SceneRenderReq(taskId, apictx.User.ID)
	if err != nil {
		repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionTask, taskId)
		return nil, err
	}

	return taskId, nil
}

func ReqestTaskWorker(ctx *repo.RepoSession, task *model.WorkerRequest, taskCollection string) error {
	taskCenterHost := conf.AppConfig.TaskCenter
	if len(taskCenterHost) < 1 {
		return errors.New("任务中心没有配置，没法执行耗时任务")
	}

	client := &http.Client{}
	bytesData, _ := json.Marshal(task)
	req, err := http.NewRequest("POST", taskCenterHost+"/execTask", bytes.NewReader(bytesData))
	if err != nil {
		return err
	}

	resp, err := client.Do(req)
	if err != nil {
		return err
	}
	if resp.StatusCode != 200 {
		return errors.New("taskCenter 请求出错")
	}

	body, _ := ioutil.ReadAll(resp.Body)

	taskRep := &model.WorkerResponse{}

	err = json.Unmarshal(body, taskRep)
	if err != nil {
		return errors.New("解析WorkerResponse错误!")
	}

	if taskRep.ErrorNo != 200 {
		return errors.New(taskRep.ErrorDesc)
	}

	//_, err = repo.RepoUpdateSetDocProps(ctx, taskCollection, task.TaskId, bson.M{"$set": bson.M{"workingId": taskRep.Result.WorkingId}})

	return nil
}
