package api

import (
	"fmt"
	"sku3dweb/bus"
	"sku3dweb/db/model"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"infish.cn/comm"
)

const ()

func RegTree(router *GinRouter) {
	router.GETJWT("/tree/user/:type", getPersonAssets)
	router.GETJWT("/tree/team/:type", getTeamAssets)
	router.GETJWT("/tree/company/:type", getCompanyAssets)
	router.GET("/tree/platform/:type", getPlatformAssets)

	router.GETJWT("/tree/categories", getUserCategory)
	router.GET("/platform/categories", getPlatformCategory)

	router.POSTJWT("/user/category", updateUserCategory)
	router.POSTJWT("/user/assetcategory", updateUserAssetCategory)
}

type AssetCate struct {
	AssetType   string   `json:"assetType"`
	CategoryIds []string `json:"categoryIds"`
}

func updateUserCategory(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	if !primitive.IsValidObjectID(apictx.User.Parent) {
		return nil, NewError("Id参数非法!")
	}
	body := &comm.DbCategory{}
	c.ShouldBindJSON(body)
	body.UserId, _ = primitive.ObjectIDFromHex(apictx.User.Parent)
	body.Scope = "sku3d"
	body.CreateTime = time.Now()
	if body.Id.IsZero() {
		body.CreateTime = time.Now()
	} else {
		body.UpdateTime = time.Now()
	}
	isSku3d := apictx.User.Parent == model.SKU3D_USERID
	if !isSku3d {
		return bus.BusUpdateUserCategory(body)
	}

	return true, bus.BusUpdateDbCategory(Sku3dDbName, body)
}

func updateUserAssetCategory(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	body := &struct {
		AssetType   string
		CategoryIds []string
	}{}
	c.ShouldBindJSON(body)
	if len(body.AssetType) < 1 {
		return nil, fmt.Errorf("参数错误")
	}

	isSku3d := apictx.User.Parent == model.SKU3D_USERID
	CurrUserScope := SCOPE_PERSONAl
	if model.USERTYPE_COMPANY == apictx.User.UserType {
		CurrUserScope = SCOPE_COMPANY
	}

	treeDb, err := bus.BusGetDbAssets(Sku3dDbName)
	if err != nil {
		return nil, err
	}

	if isSku3d { //更新数据库分类配置
		CurrUserScope = SCOPE_PLATFORM
		for _, asset := range treeDb.Assets {
			currAssetType, scope := getAssetTypeFromCollecton(asset.Collection)
			if scope != CurrUserScope || currAssetType != body.AssetType {
				continue
			}
			asset.CategoryIds = body.CategoryIds
		}
		return true, bus.BusUpdateDbAssets(Sku3dDbName, treeDb)
	}

	//更新用户自己的资产定义
	assetCate, err := bus.BusGetUserAssetCategory("sku3d", apictx.User.Parent)
	if err != nil {
		return nil, err
	}
	assetCate.Scope = "sku3d"

	for _, asset := range treeDb.Assets {
		currAssetType, scope := getAssetTypeFromCollecton(asset.Collection)
		if scope != CurrUserScope || currAssetType != body.AssetType {
			continue
		}

		defineId := asset.Id
		if len(body.CategoryIds) < 1 { //清空配置
			confs := []*comm.DbAssetCategory{}
			for _, asset2cate := range assetCate.CategoryConfs {
				if asset2cate.DbAssetId.Hex() != defineId {
					confs = append(confs, asset2cate)
				}
			}
			assetCate.CategoryConfs = confs
			continue
		}

		finded := false
		for _, asset2cate := range assetCate.CategoryConfs {
			if asset2cate.DbAssetId.Hex() == defineId {
				asset2cate.CategoryIds = body.CategoryIds
				finded = true
			}
		}
		if !finded {
			id, _ := primitive.ObjectIDFromHex(defineId)
			conf := &comm.DbAssetCategory{DbAssetId: id, CategoryIds: body.CategoryIds}
			assetCate.CategoryConfs = append(assetCate.CategoryConfs, conf)
		}
	}
	if assetCate.Id.IsZero() {
		assetCate.CreateTime = time.Now()
		assetCate.UserId, _ = primitive.ObjectIDFromHex(apictx.User.Parent)
	} else {
		assetCate.UpdateTime = time.Now()
	}

	return bus.BusUpdateUserAssetCategory(assetCate)
}

// 获取分类配置
func getUserCategory(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	userId := apictx.User.Parent
	ret, err := bus.BusGetDbDetail(userId)
	if err != nil {
		return nil, err
	}

	fmt.Sprintln(ret.Db, ret.UserCategory, ret.UserAssetCategory)

	out := map[string]interface{}{}

	db := ret.Db

	out["DbCategory"] = db.Categories.Categories
	out["CusCategory"] = ret.UserCategory.Categories

	dbAsset2Category := []*AssetCate{}

	assetMap := map[string]string{}
	assetId2CollMap := map[string]string{}
	//这里会拿到所有类型的分类包括 企业账号的，个人账号的，由于企业账号和个人账号的分类是自定义的分类这里展示过滤掉
	//dbAsset2Category 这里面只存储平台的自定义的分类，每种类型只返回一个数据，因为都是一样的
	assetTypeCateMap := map[string]bool{}

	for _, s := range db.Assets {
		assetType, scope := getAssetType(s.Collection)
		if len(assetType) > 0 {
			assetMap[s.Id] = assetType
			assetId2CollMap[s.Id] = s.Collection
			if !assetTypeCateMap[assetType] && scope == "platform" {
				assetTypeCateMap[assetType] = true
				if assetType == "mat2d" {
					assetType = "fabric2d"
				}
				if assetType == "mat" || assetType == "mats" {
					assetType = "fabric3d"
				}
				dbAsset2Category = append(dbAsset2Category, &AssetCate{AssetType: assetType, CategoryIds: s.CategoryIds})
			}
		}
	}
	out["DbAsset2Category"] = dbAsset2Category

	userAsset2Category := []*AssetCate{}
	uAssetType2CateMap := map[string]bool{}

	if ret.UserAssetCategory.CategoryConfs != nil {
		for _, s := range ret.UserAssetCategory.CategoryConfs {
			assetType := assetMap[s.DbAssetId.Hex()]
			if len(assetType) > 0 && !uAssetType2CateMap[assetType] {
				uAssetType2CateMap[assetType] = true
				if assetType == "mat2d" {
					assetType = "fabric2d"
				}
				if assetType == "mat" || assetType == "mats" {
					assetType = "fabric3d"
				}
				userAsset2Category = append(userAsset2Category, &AssetCate{AssetType: assetType, CategoryIds: s.CategoryIds})
			}
		}
	}
	out["UserAsset2Category"] = userAsset2Category

	return out, err
}

func getPlatformCategory(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	ret, err := bus.BusGetDbDetail("")
	if err != nil {
		return nil, err
	}
	out := map[string]interface{}{}

	db := ret.Db
	out["DbCategory"] = db.Categories.Categories

	dbAsset2Category := []*AssetCate{}

	assetMap := map[string]string{}
	assetId2CollMap := map[string]string{}
	//这里会拿到所有类型的分类包括 企业账号的，个人账号的，由于企业账号和个人账号的分类是自定义的分类这里展示过滤掉
	//dbAsset2Category 这里面只存储平台的自定义的分类，每种类型只返回一个数据，因为都是一样的
	assetTypeCateMap := map[string]bool{}

	for _, s := range db.Assets {
		assetType, scope := getAssetType(s.Collection)
		if len(assetType) > 0 {
			assetMap[s.Id] = assetType
			assetId2CollMap[s.Id] = s.Collection
			if !assetTypeCateMap[assetType] && scope == "platform" {
				assetTypeCateMap[assetType] = true
				if assetType == "mat2d" {
					assetType = "fabric2d"
				}
				if assetType == "mat" || assetType == "mats" {
					assetType = "fabric3d"
				}
				dbAsset2Category = append(dbAsset2Category, &AssetCate{AssetType: assetType, CategoryIds: s.CategoryIds})
			}
		}
	}
	out["DbAsset2Category"] = dbAsset2Category

	return out, err
}

func getCollect(c *gin.Context, apictx *ApiSession, scope string) string {
	assetType := c.Param("type")
	isCompany := model.USERTYPE_COMPANY == apictx.User.UserType
	isSku3d := apictx.User.Parent == model.SKU3D_USERID
	return getAssetTypeCollectin(isCompany, isSku3d, scope, assetType)
}

func getPersonAssets(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)
	// uid := apictx.User.Parent
	// apictx.User.Role
	coll := getCollect(c, apictx, "user")
	fmt.Println(apictx.User, page, size, query, coll)
	if len(coll) < 1 {
		return nil, fmt.Errorf("获取数据定义失败")
	}
	query["userId"], _ = primitive.ObjectIDFromHex(apictx.User.ID)
	projs := []string{"name", "thumbnail", "cusNum", "assetType", "assetState", "userInfo", "cusCategories", "categories", "enable", "createTime", "config", "transTask", "shadowTask"}
	if !isAssetPack(c.Param("type")) {
		projs = append(projs, "source")
	}

	req := &bus.TreeListReq{
		DbName:     Sku3dDbName,
		Collection: coll,
		Page:       int(page),
		Size:       int(size),
		Query:      query,
		Project:    projs,
	}
	return bus.BusGetAssetList(req)
}

func getTeamAssets(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	page, size, query := UtilQueryPageSize(c)
	coll := getCollect(c, apictx, "team")

	if len(coll) < 1 {
		return nil, fmt.Errorf("获取数据定义失败")
	}
	if query["ownerId"] == nil {
		return nil, fmt.Errorf("部门ID不能为空")
	}
	// query["userId"], _ = primitive.ObjectIDFromHex(apictx.User.ID)
	projs := []string{"name", "thumbnail", "cusNum", "assetType", "assetState", "userInfo", "cusCategories", "categories", "enable", "createTime", "config", "transTask", "shadowTask"}
	if !isAssetPack(c.Param("type")) {
		projs = append(projs, "source")
	}

	req := &bus.TreeListReq{
		DbName:     Sku3dDbName,
		Collection: coll,
		Page:       int(page),
		Size:       int(size),
		Query:      query,
		Project:    projs,
	}
	return bus.BusGetAssetList(req)
}

func getCompanyAssets(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)
	coll := getCollect(c, apictx, "company")

	if len(coll) < 1 {
		return nil, fmt.Errorf("获取数据定义失败")
	}

	isSku3d := apictx.User.Parent == model.SKU3D_USERID
	if !isSku3d {
		//添加企业号
		query["ownerId"], _ = primitive.ObjectIDFromHex(apictx.User.Parent)
	}

	// query["userId"], _ = primitive.ObjectIDFromHex(apictx.User.Parent)
	projs := []string{"name", "thumbnail", "cusNum", "assetType", "assetState", "userInfo", "cusCategories", "categories", "enable", "createTime", "config", "transTask", "shadowTask"}
	if !isAssetPack(c.Param("type")) {
		projs = append(projs, "source")
	}
	fmt.Println("数据库对应表名:", coll)

	req := &bus.TreeListReq{
		DbName:     Sku3dDbName,
		Collection: coll,
		Page:       int(page),
		Size:       int(size),
		Query:      query,
		Project:    projs,
	}

	return bus.BusGetAssetList(req)
}

func getPlatformAssets(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)

	assetType := c.Param("type")
	coll := getAssetTypeCollectin(true, true, "company", assetType)
	if len(coll) < 1 {
		return nil, fmt.Errorf("获取数据定义失败")
	}
	query["enable"] = true
	projs := []string{"name", "thumbnail", "cusNum", "assetType", "enable", "assetState", "categories", "createTime", "config", "transTask", "shadowTask"}
	if !isAssetPack(c.Param("type")) {
		projs = append(projs, "source")
	}
	fmt.Println("platform数据库对应表名:", coll)

	req := &bus.TreeListReq{
		DbName:     Sku3dDbName,
		Collection: coll,
		Page:       int(page),
		Size:       int(size),
		Query:      query,
		Project:    projs,
	}
	return bus.BusGetAssetList(req)
}
