package model

import (
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

type Transform struct {
}

type Vect2 [2]float64
type Vect3 [3]float64
type Vect4 [4]float64

type VectXYWH struct {
	Y float64 `bson:"y" json:"y"`
	X float64 `bson:"x" json:"x"`
	W float64 `bson:"w" json:"w"`
	H float64 `bson:"h" json:"h"`
}

type DynamicProp struct {
	Label string  `bson:"label"`
	Name  string  `bson:"name"`
	Value float32 `bson:"value"`
	Type  string  `bson:"type"`
}

type ComponentTransform struct {
	Translate []float32 `bson:"translate"`
	Rotate    []float32 `bson:"rotate"`
}

type ComponentMaterial struct {
	Thumbnail *OssType       `bson:"thumbnail,omitempty"`
	Name      string         `bson:"name,omitempty"`
	Id        string         `bson:"id"`
	Offset    *Vect2         `bson:"offset"`
	Repeat    *Vect2         `bson:"repeat"`
	Props     []*DynamicProp `bson:"props"`
}

type ComponentImage struct {
	Thumbnail        *OssType `bson:"thumbnail"`
	Uvpos            Vect2    `bson:"uvpos"`
	Uvscale          Vect2    `bson:"uvscale"`
	FillMode         string   `bson:"fillMode"`
	ProcessType      string   `bson:"processType"`
	BasicColor       Vect3    `bson:"basicColor"`
	AotuFactor       float32  `bson:"aotuFactor"`
	RoughFactor      float32  `bson:"roughFactor"`
	ReflectionFactor float32  `bson:"reflectionFactor"`
	MetalFactor      float32  `bson:"metalFactor"`
}

type ProductComponentUv struct {
	Img       *OssType           `bson:"img"`
	Transform ComponentTransform `bson:"transform"`
}
type ProductComponent struct {
	Id           string              `bson:"id"`
	Uv           *ProductComponentUv `bson:"uv"`
	BaseMaterial *ComponentMaterial  `bson:"baseMaterial"`
	Images       []*ComponentImage   `bson:"images"`
}

type SceneProduct struct {
	Id        string     `bson:"id,omitempty"`
	Name      string     `bson:"name,omitempty"`
	Type      string     `bson:"type,omitempty"`
	Locked    bool       `bson:"locked,omitempty"`
	Visible   bool       `bson:"visible,omitempty"`
	Transform *Transform `bson:"transform,omitempty"`
}
type Product struct {
	Id         string              `bson:"id,omitempty"`
	Name       string              `bson:"name,omitempty"` //名字唯一
	Thumbnail  OssType             `bson:"thumbnail,omitempty"`
	CreateTime time.Time           `bson:"createTime,omitempty"`
	Components []*ProductComponent `bson:"components,omitempty"`
}

type SceneOfflineRender struct {
	TaskId string  `bson:"taskId,omitempty"`
	Image  OssType `bson:"image,omitempty"`
}

type Scene struct {
	Id         string                `bson:"id,omitempty"`
	Name       string                `bson:"name,omitempty"` //名字唯一
	CreateTime time.Time             `bson:"createTime,omitempty"`
	Thumbnail  OssType               `bson:"thumbnail,omitempty"`
	Renders    []*SceneOfflineRender `bson:"renders"`
	Product    []*SceneProduct       `bson:"renders"`
}

type Project struct {
	Id         primitive.ObjectID `bson:"_id,omitempty"`
	Name       string             `bson:"name,omitempty"`
	Thumbnail  OssType            `bson:"thumbnail"`
	CreateTime time.Time          `bson:"createTime,omitempty"`
	UserId     primitive.ObjectID `bson:"userId,omitempty"`
	Products   []*Product         `bson:"products"`
	Scenes     []*Scene           `bson:"scenes"`
}

//declare interface DesignConfig {

// 	scenes: {
// 	  lights: {
// 		id: string;
// 		name: string;
// 		type: string;
// 	  }[];
// 	}[];
//}
